{Written by Attila Szomor, his e-mail: aszomor@dravanet.hu}
{$R-,W-,S-,A-}

unit ODBCConnectionUnit;

interface

uses
  DBXpress, ODBCIntf;

type
  (*
    This is the third step, called by THCIBSQLDriver.getSQLConnection
  *)
  TODBCISQLConnection = class(TInterfacedObject, ISQLConnection)
    constructor Create;
    destructor Destroy; override;
    function connect(ServerName: PChar; UserName: PChar; Password: PChar): SQLResult; stdcall;
    function disconnect: SQLResult; stdcall;
    function getSQLCommand(out pComm: ISQLCommand): SQLResult; stdcall;
    function getSQLMetaData(out pMetaData: ISQLMetaData): SQLResult; stdcall;
    function SetOption(eConnectOption: TSQLConnectionOption; lValue: LongInt): SQLResult; stdcall;
    function GetOption(eDOption: TSQLConnectionOption; PropValue: Pointer; MaxLength: SmallInt; out Length: SmallInt): SQLResult; stdcall;
    function beginTransaction(TranID: LongWord): SQLResult; stdcall;
    function commit(TranID: LongWord): SQLResult; stdcall;
    function rollback(TranID: LongWord): SQLResult; stdcall;
    function getErrorMessage(Error: PChar): SQLResult; overload; stdcall;
    function getErrorMessageLen(out ErrorLen: SmallInt): SQLResult; stdcall;
  private
    SQL_hENV: lpVoid;
    SQL_hDBC: lpVoid;
    // SQL_hSTMT: lpVoid;
    SQLBlobSize: Cardinal;
    SQLTransIsolationLevel: Cardinal;
    SQLTraceCallbackInfo: Integer;            //TSQlConnection.FTraceClientData
    SQLTraceCallbackEvent: TSQLCallbackEvent; //TSQlConnection.FTraceCallbackEvent
  end;

implementation

Uses
  SysUtils, ODBCDebug, ODBCMetaData;

constructor TODBCISQLConnection.Create;
begin
  inherited Create;
  SQL_hENV:= SQL_NULL_HENV;
  SQL_hDBC:= SQL_NULL_HDBC;
  SQLBlobSize:= Cardinal(-1);
  SQLTraceCallbackInfo:= 0;
  SQLTraceCallbackEvent:= NIL;
  DebugMessageBox('The 3rd step -> TODBCISQLConnection.Create', 'CALLED');
end;

destructor TODBCISQLConnection.Destroy;
begin
  if SQL_hDBC <> SQL_NULL_HDBC then begin
    SQLFreeConnect(SQL_hDBC);
    DebugMessageBox('FreeConnect in TODBCISQLConnection.Destroy', 'SQL_SUCCESS');
  end;
  SQL_hDBC:= SQL_NULL_HDBC;
  DebugMessageBox('TODBCISQLConnection.Destroy', 'CALLED');
end;


function TODBCISQLConnection.connect(ServerName: PChar; UserName: PChar; Password: PChar): SQLResult;
var
  szDebugMsg: packed array [0..2048] of char;
begin
  Result:= DBXERR_INVALIDUSRPASS;
  StrCopy(@szDebugMsg[0], 'TODBCISQLConnection.connect');
  StrCat(@szDebugMsg[0], NewLine);
  StrCat(@szDebugMsg[0], ServerName);
  StrCat(@szDebugMsg[0], NewLine);
  StrCat(@szDebugMsg[0], UserName);
  StrCat(@szDebugMsg[0], NewLine);
  StrCat(@szDebugMsg[0], Password);
  StrCat(@szDebugMsg[0], NewLine);
  if SQL_hENV = SQL_NULL_HENV then begin
    StrCat(@szDebugMsg[0], 'SQL_NULL_HENV');
    StrCat(@szDebugMsg[0], NewLine);
  end;
  DebugMessageBox(@szDebugMsg[0], 'CALLED');
  if SQL_hENV <> SQL_NULL_HENV then begin
    if SQL_OK(SQLAllocConnect(SQL_hENV, SQL_hDBC), True) then begin
      DebugMessageBox('SQLAllocConnect in TODBCISQLConnection.connect', 'SQL_SUCCESS');
      SQLSetConnectAttr(SQL_hDBC, SQL_ATTR_ODBC_CURSORS, SQL_CUR_USE_ODBC, 0);
      Result:= SQLConnect(SQL_hDBC, ServerName, StrLen(ServerName), UserName, SQL_NTS, Password, SQL_NTS);
      if SQL_OK(Result, True) then begin
        DebugMessageBox('SQLConnect in TODBCISQLConnection.connect', 'SQL_SUCCESS');
      end else begin
        DebugMessageBox('SQLConnect in TODBCISQLConnection.connect', 'SQL_ERROR');
        SQLErrorProcess(SQL_hENV, SQL_hDBC, SQL_NULL_HSTMT);
        disconnect;
      end;
    end else begin
      DebugMessageBox('SQLAllocConnect in TODBCISQLConnection.connect', 'SQL_ERROR');
    end;
  end;
end;

function TODBCISQLConnection.disconnect: SQLResult;
begin
  if SQL_hDBC <> SQL_NULL_HDBC then begin
    {Result:=} SQLDisconnect(SQL_hDBC);
    SQLFreeConnect(SQL_hDBC);
    SQL_hDBC:= SQL_NULL_HDBC;
    DebugMessageBox('FreeConnect in TODBCISQLConnection.disconnect', 'SQL_SUCCESS');
  end;
  Result:= SQL_SUCCESS;
end;

function TODBCISQLConnection.getSQLCommand(out pComm: ISQLCommand): SQLResult;
begin
  DebugMessageBox('TODBCISQLConnection.getSQLCommand', 'DBXERR_NOMEMORY');
  Result:= DBXERR_NOMEMORY {SQL_SUCCESS};
end;

function TODBCISQLConnection.getSQLMetaData(out pMetaData: ISQLMetaData): SQLResult;
begin
  pMetaData:= TODBCISQLMetaData.Create;
  if Assigned(pMetaData) then begin
    Result:= SQL_SUCCESS;
  end else begin
    Result:= DBXERR_NOMEMORY;
  end;
end;

// TSQLConnectionOption = (
//   eConnAutoCommit, eConnBlockingMode, eConnBlobSize, eConnRoleName,
//   eConnWaitOnLocks, eConnCommitRetain, eConnTxnIsoLevel,
//   eConnNativeHandle, eConnServerVersion, eConnCallBack, eConnHostName,
//   eConnDatabaseName, eConnCallBackInfo, eConnObjectMode, 
//   eConnMaxActiveConnection, eConnServerCharSet, eConnSqlDialect
// );
function TODBCISQLConnection.SetOption(eConnectOption: TSQLConnectionOption; lValue: LongInt): SQLResult;
var
  szDebugMsg: packed array [0..2048] of char;
begin
  StrCopy(@szDebugMsg[0], 'TODBCISQLConnection.SetOption(');
  StrPCopy(@szDebugMsg[StrLen(@szDebugMsg[0])], IntToStr(integer(eConnectOption)));
  StrCat(@szDebugMsg[0], ')');
  StrCat(@szDebugMsg[0], NewLine);
  Result:= SQL_SUCCESS;
  case eConnectOption of
    eConnNativeHandle: begin
      SQL_hENV:= lpVoid(lValue);
      if SQL_hENV = SQL_NULL_HENV then begin
        StrCat(@szDebugMsg[0],'SQL_NULL_HENV');
      end else begin
        StrCat(@szDebugMsg[0],'SQL_HENV');
      end;
      StrCat(@szDebugMsg[0], NewLine);
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    eConnBlobSize: begin
      SQLBlobSize:= lValue;
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    eConnTxnIsoLevel: begin
      SQLTransIsolationLevel:= LValue;
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    eConnCallBack: begin
      SQLTraceCallbackEvent:= TSQLCallbackEvent(lValue);
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    eConnCallBackInfo: begin
      SQLTraceCallbackInfo:= lValue;
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    else begin
      Result:= DBXERR_NOTSUPPORTED;
      DebugMessageBox(@szDebugMsg[0], 'DBXERR_NOTSUPPORTED');
    end;
  end;
end;

// TSQLConnectionOption = (
//   eConnAutoCommit, eConnBlockingMode, eConnBlobSize, eConnRoleName,
//   eConnWaitOnLocks, eConnCommitRetain, eConnTxnIsoLevel,
//   eConnNativeHandle, eConnServerVersion, eConnCallBack, eConnHostName,
//   eConnDatabaseName, eConnCallBackInfo, eConnObjectMode, 
//   eConnMaxActiveConnection, eConnServerCharSet, eConnSqlDialect
// );
function TODBCISQLConnection.GetOption(eDOption: TSQLConnectionOption; PropValue: Pointer; MaxLength: SmallInt; out Length: SmallInt): SQLResult;
var
  szDebugMsg: packed array [0..2048] of char;
begin
  StrCopy(@szDebugMsg[0], 'TODBCISQLConnection.GetOption(');
  StrPCopy(@szDebugMsg[StrLen(@szDebugMsg[0])], IntToStr(integer(eDOption)));
  StrCat(@szDebugMsg[0], ')');
  StrCat(@szDebugMsg[0], NewLine);
  if Assigned(PropValue) then begin
    Result:= SQL_SUCCESS;
    case eDOption of
      eConnNativeHandle: begin
        lpVoid(PropValue^):= SQL_hENV;
        Length:= SizeOf(lpVoid);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      eConnBlobSize: begin
        Cardinal(PropValue^):= SQLBlobSize;
        Length:= SizeOf(Cardinal);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      eConnTxnIsoLevel: begin
        Cardinal(PropValue^):= SQLTransIsolationLevel;
        Length:= SizeOf(Cardinal);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      eConnCallBack: begin
        TSQLCallbackEvent(PRopValue^):= SQLTraceCallbackEvent;
        Length:= SizeOf(Pointer);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      eConnCallBackInfo: begin
        Integer(PRopValue^):= SQLTraceCallbackInfo;
        Length:= SizeOf(Integer);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      eConnMaxActiveConnection: begin
        Integer(PropValue^):= 1;
        Length:= SizeOf(Integer);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      else begin
        DebugMessageBox(@szDebugMsg[0], 'DBXERR_NOTSUPPORTED');
        Result:= DBXERR_NOTSUPPORTED;
      end;
    end;
  end else begin
    DebugMessageBox(@szDebugMsg[0], 'DBXERR_NOMEMORY');
    Result:= DBXERR_NOMEMORY;
  end;
end;

function TODBCISQLConnection.beginTransaction(TranID: LongWord): SQLResult;
begin
  Result:= DBXERR_NOTSUPPORTED;{SQLTransact(SQL_hENV, SQL_hDBC, SQL_COMMIT);}  
  DebugMessageBox('TODBCISQLConnection.beginTransaction', 'DBXERR_NOTSUPPORTED');
end;

function TODBCISQLConnection.commit(TranID: LongWord): SQLResult;
begin
  Result:= DBXERR_NOTSUPPORTED;{SQLTransact(SQL_hENV, SQL_hDBC, SQL_COMMIT);}
  DebugMessageBox('TODBCISQLConnection.commit', 'DBXERR_NOTSUPPORTED');
end;

function TODBCISQLConnection.rollback(TranID: LongWord): SQLResult;
begin
  Result:= DBXERR_NOTSUPPORTED;{SQLTransact(SQL_hENV, SQL_hDBC, SQL_ROLLBACK);}  
  DebugMessageBox('TODBCISQLConnection.rollback', 'DBXERR_NOTSUPPORTED');
end;

function TODBCISQLConnection.getErrorMessage(Error: PChar): SQLResult;
begin
  if Assigned(Error) then
    Error^:= #0;
  Result:= SQL_SUCCESS;
end;

function TODBCISQLConnection.getErrorMessageLen(out ErrorLen: SmallInt): SQLResult;
begin
  ErrorLen:= 0;
  Result:= SQL_SUCCESS;
end;

end.
