{Written by Attila Szomor, his e-mail: aszomor@dravanet.hu}
{$R-,W-,S-,A-}

unit ODBCDriverUnit;

interface

uses
  DBXpress;

  (*
    This is the first step, called by TSQLConnection.DoConnect in Borland's SqlExpr
      VendorLib: dbxdrivers.ini->[clxODBC]->VendorLib
      SResourceFile: dbxconnections.ini->[clxODBC]->ErrorResourceFile
  *)
  function getSQLDriverODBC(VendorLib, SResourceFile: PChar; out pDriver: ISQLDriver): SQLResult; stdcall; export;

implementation

uses
  SysUtils, ODBCDebug, ODBCIntf, ODBCConnectionUnit;

type
  (*
    This is the second step, called by getSQLDriver
  *)
  TODBCISQLDriver = class(TInterfacedObject, ISQLDriver)
    constructor Create;
    destructor Destroy; override;
    function getSQLConnection(out pConn: ISQLConnection): SQLResult; stdcall;
    function SetOption(eDOption: TSQLDriverOption; PropValue: LongInt): SQLResult; stdcall;
    function GetOption(eDOption: TSQLDriverOption; PropValue: Pointer; MaxLength: SmallInt; out Length: SmallInt): SQLResult; stdcall;
  private
    SQL_hENV: lpVoid;
  end;

(* ---------------------------------------------------------------------------
  This is the first step, called by TSQLConnection.DoConnect
*)
function getSQLDriverODBC(VendorLib, SResourceFile: PChar; out pDriver: ISQLDriver): SQLResult;
var
  szDebugMsg: packed array [0..1024] of char;
begin
  StrCopy(@szDebugMsg[0], 'The 1st step -> getSQLDriverODBC');
  StrCat(@szDebugMsg[0], NewLine);
  StrCat(@szDebugMsg[0], VendorLib);
  StrCat(@szDebugMsg[0], ' with ');
  StrCat(@szDebugMsg[0], SResourceFile);
  StrCat(@szDebugMsg[0], NewLine);
  //if LoadODBC_DLL(VendorLib) then begin
    DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    pDriver:= TODBCISQLDriver.Create;
    if Assigned(pDriver) then begin
      Result:= SQL_SUCCESS;
    end else begin
      Result:= DBXERR_NOMEMORY;
    end;
  //end else begin
  //  DebugMessageBox(@szDebugMsg[0], 'DBXERR_INVALIDUSRPASS');
  //  Result:= DBXERR_INVALIDUSRPASS;
  //end;
end;

(* ---------------------------------------------------------------------------
  This is the second step, called by getSQLDriverODBC
*)
constructor TODBCISQLDriver.Create;
begin
  inherited Create;
  DebugMessageBox('The 2nd step -> TODBCISQLDriver.Create', 'SQL_SUCCESS');
  if SQLAllocEnv(SQL_hENV) = ODBCIntf.SQL_SUCCESS then begin
    SQLSetEnvAttr(SQL_hENV, SQL_ATTR_ODBC_VERSION, SQL_OV_ODBC3, 0);
    DebugMessageBox('AllocEnv in TODBCISQLDriver.Create', 'SQL_SUCCESS');
  end else begin
    SQL_hENV:= SQL_NULL_HENV;
    DebugMessageBox('AllocEnv in TODBCISQLDriver.Create', 'SQL_ERROR');
  end;
end;

destructor TODBCISQLDriver.Destroy;
begin
  if SQL_hENV <> SQL_NULL_HENV then begin
    SQLFreeEnv(SQL_hENV);
    DebugMessageBox('FreeEnv in TODBCISQLDriver.Destroy', 'SQL_SUCCESS');
  end else begin
    DebugMessageBox('FreeEnv in TODBCISQLDriver.Destroy', 'SQL_ERROR');
  end;
  SQL_hENV:= SQL_NULL_HENV;
  inherited Destroy;
end;

function TODBCISQLDriver.getSQLConnection(out pConn: ISQLConnection): SQLResult;
begin
  pConn:= TODBCISQLConnection.Create;
  if Assigned(pConn) then begin
    Result:= pConn.SetOption(eConnNativeHandle, longInt(SQL_hENV));
    if Result <> SQLResult(SQL_SUCCESS) then begin
      pConn:= NIL;
    end;
  end else begin
    Result:= DBXERR_NOMEMORY;
  end;
end;

// TSQLDriverOption = (
//   eDrvBlobSize, eDrvCallBack, eDrvCallBackInfo, eDrvRestrict
// );
function TODBCISQLDriver.SetOption(eDOption: TSQLDriverOption; PropValue: LongInt): SQLResult;
begin
  case eDOption of
    eDrvRestrict: begin
      Result:= SQL_SUCCESS;
    end;
    else begin
      DebugMessageBox('TODBCISQLDriver.SetOption', 'DBXERR_NOTSUPPORTED');
      Result:= DBXERR_NOTSUPPORTED;
    end;
  end;
end;

// TSQLDriverOption = (
//   eDrvBlobSize, eDrvCallBack, eDrvCallBackInfo, eDrvRestrict
// );
function TODBCISQLDriver.GetOption(eDOption: TSQLDriverOption; PropValue: Pointer; MaxLength: SmallInt; out Length: SmallInt): SQLResult;
begin
  DebugMessageBox('TODBCISQLDriver.GetOption', 'DBXERR_NOTSUPPORTED');
  Result:= DBXERR_NOTSUPPORTED;
end;

type
  TgetSQLDriver = function(VendorLib, SResourceFile: PChar; out pDriver): SQLResult; stdcall;

initialization
  DBXpress.getSQLDriver:= TgetSQLDriver(@getSQLDriverODBC);
  LoadODBC_DLL(NIL); // will go into the getSQLDriverODBC.

finalization
  UnloadODBC_DLL;  // already there is in ODBCIntf.
end.
