{Written by Attila Szomor, his e-mail: aszomor@dravanet.hu}
{$R-,W-,S-,A-}

unit ODBCMetaData;

interface

uses
  DBXpress, ODBCIntf;

type
  (*
    This is the forth step, called by TODBCISQLConnection.getSQLMetaData
  *)
  TODBCISQLMetaData = class(TInterfacedObject, ISQLMetaData)
    constructor Create;
    destructor Destroy; override;
    function SetOption(eDOption: TSQLMetaDataOption; PropValue: LongInt): SQLResult; stdcall;
    function GetOption(eDOption: TSQLMetaDataOption; PropValue: Pointer; MaxLength: SmallInt; out Length: SmallInt): SQLResult; stdcall;
    function getObjectList(eObjType: TSQLObjectType; out Cursor: ISQLCursor): SQLResult; stdcall;
    function getTables(TableName: PChar; TableType: LongWord; out Cursor: ISQLCursor): SQLResult; stdcall;
    function getProcedures(ProcedureName: PChar; ProcType: LongWord; out Cursor: ISQLCursor): SQLResult; stdcall;
    function getColumns(TableName: PChar; ColumnName: PChar; ColType: LongWord; Out Cursor: ISQLCursor): SQLResult; stdcall;
    function getProcedureParams(ProcName: PChar; ParamName: PChar; out Cursor: ISQLCursor): SQLResult; stdcall;
    function getIndices(TableName: PChar; IndexType: LongWord; out Cursor: ISQLCursor): SQLResult; stdcall;
    function getErrorMessage(Error: PChar): SQLResult; overload; stdcall;
    function getErrorMessageLen(out ErrorLen: SmallInt): SQLResult; stdcall;
  private
    szMetaSchemaName: packed array [0..MAXNAMELEN] of char;
  end;

implementation

uses
  SysUtils, ODBCDebug;

constructor TODBCISQLMetaData.Create;
begin
  inherited Create;
  szMetaSchemaName[0]:= #0;
  DebugMessageBox('The 4th step -> TODBCISQLMetaData.Create', 'CALLED');
end;

destructor TODBCISQLMetaData.Destroy;
begin
  DebugMessageBox('TODBCISQLMetaData.Destroy', 'CALLED');
  inherited Destroy;
end;

// TSQLMetaDataOption = (
//   eMetaCatalogName, eMetaSchemaName, eMetaDatabaseName,
//   eMetaDatabaseVersion, eMetaTransactionIsoLevel, eMetaSupportsTransaction,
//   eMetaMaxObjectNameLength, eMetaMaxColumnsInTable, eMetaMaxColumnsInSelect,
//   eMetaMaxRowSize, eMetaMaxSQLLength, eMetaObjectQuoteChar,
//   eMetaSQLEscapeChar, eMetaProcSupportsCursor, eMetaProcSupportsCursors, 
//   eMetaSupportsTransactions
// );
function TODBCISQLMetaData.SetOption(eDOption: TSQLMetaDataOption; PropValue: LongInt): SQLResult;
var
  szDebugMsg: packed array [0..2048] of char;
begin
  StrCopy(@szDebugMsg[0], 'TODBCISQLMetaData.SetOption(');
  StrPCopy(@szDebugMsg[StrLen(@szDebugMsg[0])], IntToStr(integer(eDOption)));
  StrCat(@szDebugMsg[0], ')');
  StrCat(@szDebugMsg[0], NewLine);
  Result:= SQL_SUCCESS;
  case eDOption of
    eMetaSchemaName: begin
      StrLCopy(@szMetaSchemaName[0], PChar(PropValue), MAXNAMELEN);
      StrCat(@szDebugMsg[0], @szMetaSchemaName[0]);
      StrCat(@szDebugMsg[0], NewLine);
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    eMetaObjectQuoteChar: begin
      StrCat(@szDebugMsg[0], 'QuoteChar is ');
      szDebugMsg[StrLen(@szDebugMsg[0])+1]:= #0;
      szDebugMsg[StrLen(@szDebugMsg[0])]:= PChar(PropValue)^;
      StrCat(@szDebugMsg[0], NewLine);
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    eMetaSupportsTransaction, eMetaSupportsTransactions: begin
      StrCat(@szDebugMsg[0], 'Transactions are not supported in MetaData');
      StrCat(@szDebugMsg[0], NewLine);
      DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
    end;
    else begin
      DebugMessageBox(@szDebugMsg[0], 'DBXERR_NOTSUPPORTED');
      Result:= DBXERR_NOTSUPPORTED;
    end;
  end;
end;

// TSQLMetaDataOption = (
//   eMetaCatalogName, eMetaSchemaName, eMetaDatabaseName,
//   eMetaDatabaseVersion, eMetaTransactionIsoLevel, eMetaSupportsTransaction,
//   eMetaMaxObjectNameLength, eMetaMaxColumnsInTable, eMetaMaxColumnsInSelect,
//   eMetaMaxRowSize, eMetaMaxSQLLength, eMetaObjectQuoteChar,
//   eMetaSQLEscapeChar, eMetaProcSupportsCursor, eMetaProcSupportsCursors, 
//   eMetaSupportsTransactions
// );
function TODBCISQLMetaData.GetOption(eDOption: TSQLMetaDataOption; PropValue: Pointer; MaxLength: SmallInt; out Length: SmallInt): SQLResult;
var
  szDebugMsg: packed array [0..2048] of char;
begin
  StrCopy(@szDebugMsg[0], 'TODBCISQLMetaData.GetOption(');
  StrPCopy(@szDebugMsg[StrLen(@szDebugMsg[0])], IntToStr(integer(eDOption)));
  StrCat(@szDebugMsg[0], ')');
  StrCat(@szDebugMsg[0], NewLine);
  if Assigned(PropValue) then begin
    Result:= SQL_SUCCESS;
    case eDOption of
      eMetaSchemaName: begin
        StrLCopy(PChar(PropValue), @szMetaSchemaName[0], MaxLength);
        Length:= StrLen(PChar(PropValue));
        StrCat(@szDebugMsg[0], PChar(PropValue));
        StrCat(@szDebugMsg[0], NewLine);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      eMetaObjectQuoteChar: begin
        Char(PropValue^):= '"';
        Length:= 1;
        StrCat(@szDebugMsg[0], 'QuoteChar is "');
        StrCat(@szDebugMsg[0], NewLine);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      eMetaSupportsTransaction, eMetaSupportsTransactions: begin
        Integer(PropValue^):= 0;
        Length:= SizeOf(Integer);
        StrCat(@szDebugMsg[0], 'Transactions are not supported in MetaData');
        StrCat(@szDebugMsg[0], NewLine);
        DebugMessageBox(@szDebugMsg[0], 'SQL_SUCCESS');
      end;
      else begin
        DebugMessageBox(@szDebugMsg[0], 'DBXERR_NOTSUPPORTED');
        Result:= DBXERR_NOTSUPPORTED;
      end;
    end;
  end else begin
    Result:= DBXERR_NOMEMORY;
  end;
end;

function TODBCISQLMetaData.getObjectList(eObjType: TSQLObjectType; out Cursor: ISQLCursor): SQLResult;
begin
  DebugMessageBox('TODBCISQLMetaData.getObjectList', 'DBXERR_NOTSUPPORTED');
  Result:= DBXERR_NOTSUPPORTED;
end;

function TODBCISQLMetaData.getTables(TableName: PChar; TableType: LongWord; out Cursor: ISQLCursor): SQLResult;
begin
  DebugMessageBox('TODBCISQLMetaData.getTables', 'DBXERR_NOTSUPPORTED');
  Result:= DBXERR_NOTSUPPORTED;
end;

function TODBCISQLMetaData.getProcedures(ProcedureName: PChar; ProcType: LongWord; out Cursor: ISQLCursor): SQLResult;
begin
  DebugMessageBox('TODBCISQLMetaData.getProcedures', 'DBXERR_NOTSUPPORTED');
  Result:= DBXERR_NOTSUPPORTED;
end;

function TODBCISQLMetaData.getColumns(TableName: PChar; ColumnName: PChar; ColType: LongWord; Out Cursor: ISQLCursor): SQLResult;
begin
  DebugMessageBox('TODBCISQLMetaData.getColumns', 'DBXERR_NOTSUPPORTED');
  Result:= DBXERR_NOTSUPPORTED;
end;

function TODBCISQLMetaData.getProcedureParams(ProcName: PChar; ParamName: PChar; out Cursor: ISQLCursor): SQLResult;
begin
  DebugMessageBox('TODBCISQLMetaData.getProcedureParams', 'DBXERR_NOTSUPPORTED');
  Result:= DBXERR_NOTSUPPORTED;
end;

function TODBCISQLMetaData.getIndices(TableName: PChar; IndexType: LongWord; out Cursor: ISQLCursor): SQLResult;
begin
  DebugMessageBox('TODBCISQLMetaData.getIndices', 'DBXERR_NOTSUPPORTED');
  Result:= DBXERR_NOTSUPPORTED;
end;

function TODBCISQLMetaData.getErrorMessage(Error: PChar): SQLResult;
begin
  if Assigned(Error) then
    Error^:= #0;
  Result:= SQL_SUCCESS;
end;

function TODBCISQLMetaData.getErrorMessageLen(out ErrorLen: SmallInt): SQLResult;
begin
  ErrorLen:= 0;
  Result:= SQL_SUCCESS;
end;

end.


